//
// A cube renderer
//
// (c) 2012 James Beau Wilkinson
//
// Licensed under the GNU General Public License v3+
//

package com.beauwilkinson.demofox;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;

import javax.microedition.khronos.opengles.GL10;

final class cube
{

 private static final int PRISM_FULL_COLOR = 0x08000;

 private static final int PRISM_HALF_COLOR = 0x04000;

 private static final float PRISM_CUBE_WIDTH = 1f;

 //@formatter:off
 private static final float VERTICES[] = 
 {
   // Back
  -PRISM_CUBE_WIDTH/2f, -PRISM_CUBE_WIDTH/2f, -PRISM_CUBE_WIDTH/2f, 
  PRISM_CUBE_WIDTH/2f, -PRISM_CUBE_WIDTH/2f, -PRISM_CUBE_WIDTH/2f,
  PRISM_CUBE_WIDTH/2f,  PRISM_CUBE_WIDTH/2f, -PRISM_CUBE_WIDTH/2f, 
  -PRISM_CUBE_WIDTH/2f, PRISM_CUBE_WIDTH/2f, -PRISM_CUBE_WIDTH/2f,

  // Front
  -PRISM_CUBE_WIDTH/2f, -PRISM_CUBE_WIDTH/2f, PRISM_CUBE_WIDTH/2f, 
  PRISM_CUBE_WIDTH/2f, -PRISM_CUBE_WIDTH/2f, PRISM_CUBE_WIDTH/2f, 
  PRISM_CUBE_WIDTH/2f, PRISM_CUBE_WIDTH/2f, PRISM_CUBE_WIDTH/2f, 
  -PRISM_CUBE_WIDTH/2f, PRISM_CUBE_WIDTH/2f, PRISM_CUBE_WIDTH/2f, 
 };

 // 6 sides * 2 triangles per side * 3 vertices per triangle = 36
 private static final int CUBE_VERTEX_COUNT = 36;
 
 // This is the specific, clockwise zig-zagging order
 // necessary for optimal rendering of the solid.
 private static final byte CUBE_VERTEX_INDICES[] = 
 { 
  0, 4, 5,  //First triangle drawn
  0, 5, 1,  //Second... etc.
  1, 5, 6, 
  1, 6, 2, 
  2, 6, 7, 
  2, 7, 3, 
  3, 7, 4, 
  3, 4, 0, 
  4, 7, 6, 
  4, 6, 5, 
  3, 0, 1, 
  3, 1, 2 
 };
 //@formatter:on

 protected static void draw(final GL10 gl)
 {
  gl.glDisable(GL10.GL_TEXTURE_2D);
  gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
  gl.glFrontFace(GL10.GL_CW);

  // 3 means dimensions: x, y, and z
  gl.glVertexPointer(3, GL10.GL_FLOAT, 0, vertexbuffer);

  // 4 means R, G, B, A here
  gl.glColorPointer(4, GL10.GL_FIXED, 0, colorbuffer);

  gl.glDrawElements(GL10.GL_TRIANGLES, CUBE_VERTEX_COUNT,
   GL10.GL_UNSIGNED_BYTE, indexbuffer);
  gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);
 }

 protected static float getPrismCubeWidth()
 {
  return PRISM_CUBE_WIDTH;
 }

 private static FloatBuffer vertexbuffer;
 private static IntBuffer colorbuffer;
 private static ByteBuffer indexbuffer;

 static
 {

  final int fullcolor = PRISM_FULL_COLOR;
  final int halfcolor = PRISM_HALF_COLOR;

  //@formatter:off
  final int colors[] = 
  { 
   halfcolor, halfcolor, halfcolor, fullcolor,
   halfcolor, halfcolor, halfcolor, fullcolor, 
   fullcolor, fullcolor, fullcolor, fullcolor, 
   fullcolor, fullcolor, fullcolor, fullcolor, 
   halfcolor, halfcolor, halfcolor, fullcolor, 
   halfcolor, halfcolor, halfcolor, fullcolor,
   fullcolor, fullcolor, fullcolor, fullcolor, 
   fullcolor, fullcolor, fullcolor, fullcolor 
  };
  //@formatter:on

  // Use of a "direct" buffer is required per the OpenGL docs
  final ByteBuffer vbb = ByteBuffer.allocateDirect(VERTICES.length * Float.SIZE
   / Byte.SIZE);
  vbb.order(ByteOrder.nativeOrder());
  vertexbuffer = vbb.asFloatBuffer();
  vertexbuffer.put(VERTICES);
  vertexbuffer.position(0);

  final ByteBuffer cbb = ByteBuffer
   .allocateDirect(colors.length * Integer.SIZE);
  cbb.order(ByteOrder.nativeOrder());
  colorbuffer = cbb.asIntBuffer();
  colorbuffer.put(colors);
  colorbuffer.position(0);
  indexbuffer = ByteBuffer.allocateDirect(CUBE_VERTEX_INDICES.length);
  indexbuffer.put(CUBE_VERTEX_INDICES);
  indexbuffer.position(0);
 }

}
