//
// A backplane renderer
//
// (c) 2012 James Beau Wilkinson
//
// Licensed under the GNU General Public License v3+
//

package com.beauwilkinson.demofox;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import javax.microedition.khronos.opengles.GL10;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLUtils;

//This plane will appear in the user's face if looking fwd/backwd 
// in the Z-dimension
final class plane
{

 private static final int PLANE_FULL_COLOR = 0x08000;
 private static final float SKY_PLANE_WIDTH = 180f;

 // This is the specific, clockwise zig-zagging order
 // necessary for optimal rendering of the plane.
 private static final byte PLANE_VERTEX_INDICES[] =
 {
  0, 1, 3, 2, 3, 1
 };

 // Two triangles * 3 vertices per triangle
 private static final int PLANE_VERTICES = 6;

 protected static void draw(GL10 gl)
 {

  if (!textureready)
  {
   textureready = true;
   loadtexture(gl, demofoxactivity.getSingleton());
  }

  gl.glEnable(GL10.GL_TEXTURE_2D);
  gl.glBindTexture(GL10.GL_TEXTURE_2D, textures[0]);
  gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
  gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);

  gl.glFrontFace(GL10.GL_CW);

  // Three dimensions are {X, Y, Z}
  gl.glVertexPointer(3, GL10.GL_FLOAT, 0, vertexbuffer);

  // 4 is {R, G, B, A}
  gl.glColorPointer(4, GL10.GL_FIXED, 0, colorbuffer);

  // 2 is {U, V}
  gl.glTexCoordPointer(2, GL10.GL_FLOAT, 0, texturebuffer);

  gl.glDrawElements(GL10.GL_TRIANGLES, PLANE_VERTICES, GL10.GL_UNSIGNED_BYTE,
   indexbuffer);

  // Disable the client state before leaving
  gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);
  gl.glDisableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
 }

 protected static void rebuild()
 {
  textureready = false;
 }

 private static void loadtexture(GL10 gl, Context context)
 {
  Bitmap bitmap;

  // Loading texture
  if (demofoxactivity.getLevel() == 1)
  {
   bitmap = BitmapFactory.decodeResource(context.getResources(),
    R.drawable.background);
  }
  else
  {
   bitmap = BitmapFactory.decodeResource(context.getResources(),
    R.drawable.other);
  }

  // Generate one texture pointer
  gl.glGenTextures(1, textures, 0);

  // ...and bind it to our array
  gl.glBindTexture(GL10.GL_TEXTURE_2D, textures[0]);

  // Use "nearest pixel" filter for sizing
  gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MIN_FILTER,
   GL10.GL_NEAREST);

  gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MAG_FILTER,
   GL10.GL_LINEAR);

  // Use GLUtils to make a two-dimensional texture image from
  // the bitmap
  GLUtils.texImage2D(GL10.GL_TEXTURE_2D, 0, bitmap, 0);

  // Allow for resource reclamation
  bitmap.recycle();
 }

 private static FloatBuffer vertexbuffer;
 private static IntBuffer colorbuffer;
 private static ByteBuffer indexbuffer;
 private static FloatBuffer texturebuffer;
 private static boolean textureready = false;
 private static int[] textures = new int[1];

 static
 {

  final int fullcolor = PLANE_FULL_COLOR;
  final float width = SKY_PLANE_WIDTH;

  //@formatter:off
  float vertices[] = 
  {
   -width, -width, -width, 
    width, -width, -width, 
    width, width, -width,
   -width, width, -width 
  };

  int colors[] = 
  {  
   fullcolor, fullcolor, fullcolor, fullcolor, 
   fullcolor, fullcolor, fullcolor, fullcolor, 
   fullcolor, fullcolor, fullcolor, fullcolor, 
   fullcolor, fullcolor, fullcolor, fullcolor 
  };
  //@formatter:on

  //
  // Per the docs:
  //
  // "Buffers to be passed to gl*Pointer() functions
  // must be direct, i.e., they must be placed on the
  // native heap where the garbage collector cannot
  // move them.
  //
  // Buffers with multi-byte datatypes (e.g., short, int, float)
  // must have their byte order set to native order"

  ByteBuffer vbb = ByteBuffer.allocateDirect(vertices.length * Float.SIZE
   / Byte.SIZE);
  vbb.order(ByteOrder.nativeOrder());
  vertexbuffer = vbb.asFloatBuffer();
  vertexbuffer.put(vertices);
  vertexbuffer.position(0);

  ByteBuffer cbb = ByteBuffer.allocateDirect(colors.length * Integer.SIZE
   / Byte.SIZE);
  cbb.order(ByteOrder.nativeOrder());
  colorbuffer = cbb.asIntBuffer();
  colorbuffer.put(colors);
  colorbuffer.position(0);

  indexbuffer = ByteBuffer.allocateDirect(PLANE_VERTEX_INDICES.length);
  indexbuffer.put(PLANE_VERTEX_INDICES);
  indexbuffer.position(0);

  //@formatter:off
  float texture[] =
  {
   0.0f, 0.0f, // bottom left
   1.0f, 0.0f, // bottom right
   1.0f, 1.0f, // top left
   0.0f, 1.0f  // top right
  };
  //@formatter:on

  ByteBuffer cbb3 = ByteBuffer.allocateDirect(texture.length * Float.SIZE
   / Byte.SIZE);

  cbb3.order(ByteOrder.nativeOrder());

  texturebuffer = cbb3.asFloatBuffer();

  texturebuffer.put(texture);

  texturebuffer.position(0);

 }

}
